pub fn export_lean_bundle(
module_name: &str,
bundle: &ObligationBundle,
) -> StringExpand description
Export a whole bundle as a single Lean 4 module.
pub fn export_lean_bundle(
module_name: &str,
bundle: &ObligationBundle,
) -> StringExport a whole bundle as a single Lean 4 module.