Density

Struct Density 

pub struct Density<W, A>
where W: HKT + 'static, A: 'static,
{ /* private fields */ }
Density Comonad — the CPS dual of Codensity.

Density<W, A> ≅ ∃S. (W S → A, W S)

This is the left Kan extension of W along itself (Lan W W A), specialised into a concrete type with existential state.

Key properties

  • extract: Requires no bounds on W — just applies the stored function.
  • fmap: Composes onto the extract function, no bounds on W.

Note: Due to Rust’s GAT limitations, DensityF does not implement HKT or Comonad. Use inherent methods instead.

Implementations

Source

impl<W, A> Density<W, A>
where W: HKT + 'static, A: 'static,

pub fn lift<S>( source: <W as HKT>::Of<S>, f: impl Fn(&<W as HKT>::Of<S>) -> A + 'static, ) -> Density<W, A>
where S: 'static, <W as HKT>::Of<S>: 'static,

Construct a Density from a source value and an extract function.

pub fn extract(&self) -> A

Extract the value. No bounds on W required.

pub fn fmap<B>(self, f: impl Fn(A) -> B + 'static) -> Density<W, B>
where B: 'static,

Map a function over the result. No bounds on W required.