Fold

Struct Fold 

pub struct Fold<S, A> { /* private fields */ }
A read-only multi-focus optic.

Like a Traversal but without the ability to modify.

Implementations

Source

impl<S, A> Fold<S, A>

pub fn new(fold_fn: impl Fn(&S) -> Vec<A> + 'static) -> Fold<S, A>

pub fn get_all(&self, s: &S) -> Vec<A>

pub fn fold_map<R>(&self, s: &S, f: impl Fn(A) -> R) -> R
where R: Monoid,

Map each focus to a monoid value and combine them.

pub fn any(&self, s: &S, f: impl Fn(&A) -> bool) -> bool

Check if any focus satisfies a predicate.

pub fn all(&self, s: &S, f: impl Fn(&A) -> bool) -> bool

Check if all foci satisfy a predicate.

pub fn find(&self, s: &S, f: impl Fn(&A) -> bool) -> Option<A>

Find the first focus satisfying a predicate.

pub fn length(&self, s: &S) -> usize

Count the number of foci.

pub fn then<B>(self, inner: Fold<A, B>) -> ComposedFold<S, B>
where S: 'static, A: 'static, B: 'static,

Compose with another fold for deeper read-only access.