pub struct Lean4;Expand description
Marker for the Lean 4 backend.
Implementations
Sourceimpl Lean4
impl Lean4
Sourcepub fn export_module(module_name: &str, obligations: &[Obligation]) -> String
pub fn export_module(module_name: &str, obligations: &[Obligation]) -> String
Export a list of obligations as a Lean 4 module skeleton.
Sourcepub fn export_module_with_prelude(
module_name: &str,
obligations: &[Obligation],
prelude: LeanPrelude,
) -> String
pub fn export_module_with_prelude( module_name: &str, obligations: &[Obligation], prelude: LeanPrelude, ) -> String
Export a list of obligations as a Lean 4 module skeleton with explicit prelude metadata.
Sourcepub fn export(module_name: &str, obligations: &[Obligation]) -> LeanExport
pub fn export(module_name: &str, obligations: &[Obligation]) -> LeanExport
Export a list of obligations as structured Lean metadata plus source.
Sourcepub fn export_with_prelude(
module_name: &str,
obligations: &[Obligation],
prelude: LeanPrelude,
) -> LeanExport
pub fn export_with_prelude( module_name: &str, obligations: &[Obligation], prelude: LeanPrelude, ) -> LeanExport
Export a list of obligations as structured Lean metadata plus source with explicit prelude metadata.
Sourcepub fn export_bundle(module_name: &str, bundle: &ObligationBundle) -> String
pub fn export_bundle(module_name: &str, bundle: &ObligationBundle) -> String
Export a bundle of obligations as a Lean 4 module skeleton.
Sourcepub fn export_bundle_structured(
module_name: &str,
bundle: &ObligationBundle,
) -> LeanExport
pub fn export_bundle_structured( module_name: &str, bundle: &ObligationBundle, ) -> LeanExport
Export a bundle of obligations as structured Lean metadata plus source.
Sourcepub fn project(export: &LeanExport) -> LeanProject
pub fn project(export: &LeanExport) -> LeanProject
Derive Lean package metadata from a structured export.