Positive

Struct Positive 

pub struct Positive<T> { /* private fields */ }
A numeric value guaranteed to be strictly positive (> 0).

Useful for operations that require nonzero values, such as Field::reciprocal.

Implementations

Source

impl Positive<f32>

pub fn try_new(v: f32) -> Option<Positive<f32>>

Attempt to construct a Positive from a value. Returns None if the value is not strictly positive.

pub fn get(self) -> f32

Get the inner value.

pub fn reciprocal(self) -> Positive<f32>

Safe reciprocal: always valid for positive values.

Source

impl Positive<f64>

pub fn try_new(v: f64) -> Option<Positive<f64>>

Attempt to construct a Positive from a value. Returns None if the value is not strictly positive.

pub fn get(self) -> f64

Get the inner value.

pub fn reciprocal(self) -> Positive<f64>

Safe reciprocal: always valid for positive values.

Source

impl Positive<i8>

pub fn try_new(v: i8) -> Option<Positive<i8>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> i8

Get the inner value.

Source

impl Positive<i16>

pub fn try_new(v: i16) -> Option<Positive<i16>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> i16

Get the inner value.

Source

impl Positive<i32>

pub fn try_new(v: i32) -> Option<Positive<i32>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> i32

Get the inner value.

Source

impl Positive<i64>

pub fn try_new(v: i64) -> Option<Positive<i64>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> i64

Get the inner value.

Source

impl Positive<i128>

pub fn try_new(v: i128) -> Option<Positive<i128>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> i128

Get the inner value.

Source

impl Positive<u8>

pub fn try_new(v: u8) -> Option<Positive<u8>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> u8

Get the inner value.

Source

impl Positive<u16>

pub fn try_new(v: u16) -> Option<Positive<u16>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> u16

Get the inner value.

Source

impl Positive<u32>

pub fn try_new(v: u32) -> Option<Positive<u32>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> u32

Get the inner value.

Source

impl Positive<u64>

pub fn try_new(v: u64) -> Option<Positive<u64>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> u64

Get the inner value.

Source

impl Positive<u128>

pub fn try_new(v: u128) -> Option<Positive<u128>>

Attempt to construct a Positive from a value. Returns None if the value is zero or negative.

pub fn get(self) -> u128

Get the inner value.

Source

impl Positive<u8>

pub fn from_nonzero(v: u8) -> Option<Positive<u8>>

Construct from a nonzero unsigned value. Returns None if zero.

Source

impl Positive<u16>

pub fn from_nonzero(v: u16) -> Option<Positive<u16>>

Construct from a nonzero unsigned value. Returns None if zero.

Source

impl Positive<u32>

pub fn from_nonzero(v: u32) -> Option<Positive<u32>>

Construct from a nonzero unsigned value. Returns None if zero.

Source

impl Positive<u64>

pub fn from_nonzero(v: u64) -> Option<Positive<u64>>

Construct from a nonzero unsigned value. Returns None if zero.

Source

impl Positive<u128>

pub fn from_nonzero(v: u128) -> Option<Positive<u128>>

Construct from a nonzero unsigned value. Returns None if zero.