pub struct Positive<T> { /* private fields */ }Expand description
A numeric value guaranteed to be strictly positive (> 0).
Useful for operations that require nonzero values, such as
Field::reciprocal.
Implementations